# -*- coding: utf-8 -*-
"""A full async docsplit previewer server based on Tornado and Redis"""


pkg_resources = __import__('pkg_resources')
distribution = pkg_resources.get_distribution('insight_reloaded')

#: Module version, as defined in PEP-0396.
__version__ = distribution.version



import os
import os.path
import shutil
import unittest

from tempfile import mkstemp, mkdtemp

from insight_reloaded.storage.file_system import FileSystemStorage


class FileSystemStorageTestCase(unittest.TestCase):

    def setUp(self):
        _, self.source_file = mkstemp()
        self.source_url = 'http://localhost:9200/an/random/url'
        self.destination_root = mkdtemp()
        self.prefix_url = 'http://localhost/prefix'
        self.content = 'Source file !'

        with open(self.source_file, 'wb') as f:
            f.write(self.content)

        self.storage = FileSystemStorage(self.source_url, self.destination_root,
            self.prefix_url)

    def tearDown(self):
        try:
            os.remove(self.source_file)
            shutil.rmtree(self.destination_root)
        except os.error:
            pass

    def test_prepare_and_save(self):
        filename = 'document_p1.png'
        self.storage.prepare()
        self.storage.save(self.source_file, filename)

        file_path = self.storage.get_path(filename)

        self.assertTrue(os.path.exists(file_path))

        with open(file_path) as f:
            self.assertEqual(f.read().strip(), self.content)



import json
import os

import tornadoredis
import tornado.web
import tornado.ioloop
import tornado.gen
import tornado.httpserver

from insight_reloaded.insight_settings import (
    REDIS_HOST, REDIS_PORT, REDIS_DB, SENTRY_DSN, REDIS_QUEUE_KEYS,
    DEFAULT_REDIS_QUEUE_KEY, CROP_SIZE)


try:
    from raven import Client
except ImportError:
    if SENTRY_DSN:
        SENTRY_DSN = None
        print "SENTRY_DSN is defined but raven isn't installed."


HERE = os.path.dirname(os.path.abspath(__file__))
try:
    with open(os.path.join(HERE, '../VERSION')) as f:
        VERSION = f.readlines()[0].strip()
except IOError:
    VERSION = 'N/A'


c = tornadoredis.Client(host=REDIS_HOST, port=REDIS_PORT, selected_db=REDIS_DB)
c.connect()


class MainHandler(tornado.web.RequestHandler):
    """Convert the querystring in a REDIS job JSON."""

    @tornado.web.asynchronous
    def get(self, queue):
        # If there is no argument, display the version
        if self.request.arguments == {}:
            self.write(dict(insight_reloaded="Bonjour",
                            name="insight-reloaded",
                            version=VERSION))
            self.finish()
            return

        if not queue:
            queue = DEFAULT_REDIS_QUEUE_KEY

        if queue not in REDIS_QUEUE_KEYS:
            raise tornado.web.HTTPError(404, 'Queue "%s" not found' % queue)

        self.queue = queue

        # Else process the request
        params = {'url': self.get_argument("url", None),
                  'callback': self.get_argument("callback", None),
                  }
        # Get URL
        if params['url']:
            if params['url'].startswith('/'):
                params['url'] = '%s://%s%s' % (self.request.protocol,
                                               self.request.host,
                                               params['url'])
        else:
            raise tornado.web.HTTPError(404, 'Input file not found')

        # Max number of pages to compile
        try:
            params['max_previews'] = int(self.get_argument('pages', 20))
        except:
            params['max_previews'] = 20

        try:
            params['crop'] = int(self.get_argument('crop', 0))
        except ValueError:
            params['crop'] = CROP_SIZE

        message = json.dumps(params)
        c.rpush(self.queue, message, callback=self.on_response)

    def on_response(self, response):
        self.write(dict(insight_reloaded="Job added to queue '%s'." %
                        self.queue,
                        number_in_queue=response))
        self.finish()


class StatusHandler(tornado.web.RequestHandler):
    @tornado.web.asynchronous
    def get(self, queue):
        if not queue:
            queue = DEFAULT_REDIS_QUEUE_KEY

        if queue not in REDIS_QUEUE_KEYS:
            raise tornado.web.HTTPError(404, "Queue '%s' not found" % queue)
        self.queue = queue
        c.llen(self.queue, callback=self.on_response)

    def on_response(self, response):
        self.write(dict(insight_reloaded="There is %d job in the '%s' queue." %
                        (response, self.queue),
                        number_in_queue=response))
        self.finish()


application = tornado.web.Application([
    (r"/([A-Za-z0-9_-]*)/?status", StatusHandler),
    (r"/([A-Za-z0-9_-]*)", MainHandler),
])


def main():
    if SENTRY_DSN:
        client = Client(dsn=SENTRY_DSN)
    else:
        client = None
    print "Launch tornado ioloop"
    application.listen(8888)
    try:
        tornado.ioloop.IOLoop.instance().start()
    except Exception:
        if client:
            client.get_ident(client.captureException())
        else:
            raise

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Preview documents"""

import logging
from os import path, chdir, listdir
from shutil import rmtree, copyfile
from tempfile import mkdtemp
try:
    from subprocess import STDOUT, check_output, CalledProcessError
except ImportError:  # check_output new in 2.7, so use a backport for <=2.6
    from subprocess32 import STDOUT, check_output, CalledProcessError


class PreviewException(Exception):
    pass


class DocumentPreview(object):

    def __init__(self, file_obj, callback, sizes, max_previews,
                 base_tmp_folder, storage, crop=False):
        self.file_obj = file_obj
        self.callback = callback
        self.filename = file_obj.name
        self.pages = 1  # sane default
        self.sizes = sizes
        self.max_previews = max_previews
        self.base_tmp_folder = base_tmp_folder
        self.tmp_folder = mkdtemp(dir=self.base_tmp_folder)
        self.storage = storage
        self.crop = crop

    def __unicode__(self):
        return unicode(self.doc_id)

    def create_previews(self):
        """Calls docsplit with the proper parameters."""

        self.storage.prepare()

        chdir(self.tmp_folder)
        preview_folder = path.join(self.tmp_folder, 'previews')
        self.pages = self.get_num_pages()
        if self.pages < self.max_previews:
            self.max_previews = self.pages  # max_previews <= pages
        pages = '1-%s' % self.max_previews
        logging.info(
            "Creating previews in %s for pages '%s'" % (preview_folder, pages))
        cmd = "%(cmd)s -o %(path)s -p %(pages)s -s %(sizes)s -f png %(file)s"
        try:
            check_output(cmd % {'cmd': "docsplit images",
                                'path': preview_folder,
                                'pages': pages,
                                'sizes': ','.join(self.sizes.keys()),
                                'file': self.filename},
                         shell=True,
                         stderr=STDOUT)
        except CalledProcessError, e:
            msg = u"[PREVIEW] Error while previewing %s: %s. Output: %r" % (
                self.filename, e, e.output)
            raise PreviewException(msg)
        # rename each preview using the format 'filename_{size}_p{page}.png'
        # and save all files in the "previews" folder, not in subfolders
        for size_name, size in self.sizes.iteritems():
            folder = path.join(preview_folder, size_name)  # previews/150
            for f in listdir(folder):  # filenames: filename_<page num>.png
                filename_end = f.split('_')[-1]  # <page_num>.png
                page_num = filename_end[:-4]  # remove the '.png' extension
                new_name = 'document_%s_p%s.png' % (size, page_num)
                self.storage.save(path.join(folder, f), new_name)
        if self.crop:
            self.add_crop(self.crop)

    def add_crop(self, crop):
        """Add a cropped version of the first page 'normal' sized preview"""
        logging.info(u"Cropping %s%%" % crop)
        first_page = self.storage.get_path('document_normal_p1.png')
        # copy the "to be cropped" file outside the previews folder: if the
        # crop fails, we don't want to add the full image as a cropped version
        tmp_file = path.join(self.tmp_folder, 'before_cropping.png')
        copyfile(first_page, tmp_file)
        cmd = "gm mogrify -crop 100%%x%s%% %s" % (crop, tmp_file)
        try:
            check_output(cmd, shell=True, stderr=STDOUT)
        except CalledProcessError, e:
            msg = u"[PREVIEW] Error while cropping %s: %s. Output: %r" % (
                tmp_file, e, e.output)
            raise PreviewException(msg)

        self.storage.save(tmp_file, 'document_normal_p1_cropped.ong')

    def get_num_pages(self):
        """Return the number of pages for the document"""
        logging.info(u"Getting num pages for %s" % self.filename)
        cmd = "docsplit length %s" % self.filename
        try:
            output = check_output(cmd, shell=True, stderr=STDOUT)
        except CalledProcessError, e:
            msg = u"[PREVIEW] Failed get %s num pages: %s. Output: %r" % (
                self.filename, e, e.output)
            raise PreviewException(msg)
        num_pages = output.strip()
        if num_pages.isdigit():
            return int(num_pages)
        logging.info(u"Counted %s pages" % output)
        raise PreviewException(u"[PREVIEW] output isn't a number: %s" % output)

    def cleanup(self):
        """Remove the document and its previews"""
        if self.tmp_folder is not None and path.exists(self.tmp_folder):
            logging.info(u"Removing %s" % self.tmp_folder)
            rmtree(self.tmp_folder, ignore_errors=True)



# -*- coding: utf-8 -*-
from redis import StrictRedis
import json
import requests
import os
from tempfile import NamedTemporaryFile
import mimetypes
import sys

from insight_reloaded.preview import (DocumentPreview,
                                      PreviewException)
from insight_reloaded.insight_settings import (
    REDIS_HOST, REDIS_PORT, REDIS_DB, SENTRY_DSN, REDIS_QUEUE_KEYS,
    DEFAULT_REDIS_QUEUE_KEY, ALLOWED_EXTENSIONS, TEMP_DIRECTORY, CROP_SIZE,
    DESTINATION_ROOT, PREVIEW_SIZES, PREFIX_URL, DOCVIEWER_SUFFIX,
    STORAGE_CLASS, S3_ACCESS_KEY, S3_SECRET_KEY, S3_BUCKET_NAME)
from utils import ImportStringError, resolve_name

try:
    from raven import Client
except ImportError:
    if SENTRY_DSN:
        SENTRY_DSN = None
        print "SENTRY_DSN is defined but raven isn't installed."


redis = StrictRedis(host=REDIS_HOST, port=REDIS_PORT, db=REDIS_DB)


class InsightWorkerException(Exception):
    pass


def abort(exception, requested_ressource, callback_url=None):
    """Raise an error but send it first to the callback"""
    if callback_url:
        try:
            requests.post(callback_url,
                          verify=False,
                          data={'success': False,
                                'error_message': str(exception),
                                'requested_ressource': requested_ressource})
        except requests.exceptions.ConnectionError:
            pass
    raise InsightWorkerException('%s : %s' % (requested_ressource,
                                              exception))


def main():
    if SENTRY_DSN:
        client = Client(dsn=SENTRY_DSN)
        while True:
            try:
                start_worker()
            except Exception:
                client.get_ident(client.captureException())
    else:
        while True:
            try:
                start_worker()
            except InsightWorkerException as e:
                sys.stderr.write('%s\n' % e)


def start_worker():
    # Init from CLI
    worker_command = sys.argv[0]
    argv = sys.argv[1:]
    argc = len(argv)

    if argc > 1:
        print "USAGE: %s [REDIS_QUEUE_KEY]" % worker_command
        sys.exit(1)
    elif argc == 1:
        queue = argv[0]
        if queue not in REDIS_QUEUE_KEYS:
            print "WRONG QUEUE: %s not in %s" % (queue, REDIS_QUEUE_KEYS)
            sys.exit(2)
    else:
        queue = DEFAULT_REDIS_QUEUE_KEY

    print "Launch insight worker on '%s' redis queue." % queue
    while 1:
        msg = redis.blpop(queue)  # BLPOP is blocking for the next entry
        try:
            params = json.loads(msg[1])
        except ValueError as exc:
            abort(exc, msg)

        print u"Consuming task for doc %s" % params['url']

        # Getting callback url
        if 'callback' in params:
            callback = params['callback']
        else:
            callback = None

        # Downloading file
        try:
            r = requests.get(params['url'], verify=False)  # No need
                                                           # to verify
                                                           # SSL
        except requests.exceptions.ConnectionError, e:
            abort(e, params['url'], callback)

        extensions = []
        if 'content-type' in r.headers:
            extensions += mimetypes.guess_all_extensions(
                r.headers['content-type'])

        if 'content-disposition' in r.headers:
            filename = r.headers['content-disposition'].split('filename=')[-1]
            filename = filename.strip('"').strip("'")
            extensions.extend(os.path.splitext(filename)[1:])

        # Verify if the file is accepted
        accepted_extensions = list(
            set([ext for ext in extensions if ext in ALLOWED_EXTENSIONS]))
        if len(accepted_extensions) > 0:
            extension = accepted_extensions[0]
        else:
            abort(InsightWorkerException('%s not allowed' %
                                         r.headers['content-type']),
                  params['url'], callback)

        # Creating temporary file
        file_obj = NamedTemporaryFile(suffix=extension, dir=TEMP_DIRECTORY)
        file_obj.write(r.content)
        file_obj.seek(0)

        # Settings parameters
        max_previews = params['max_previews']
        try:
            crop = int(params['crop'])
        except:
            crop = CROP_SIZE

        # Create the storage
        storage_class = resolve_name(STORAGE_CLASS)
        storage = storage_class(params['url'])

        # Here comes the document preview engine
        preview = DocumentPreview(file_obj, callback, PREVIEW_SIZES,
                                  max_previews, TEMP_DIRECTORY,
                                  storage, crop)
        try:
            preview.create_previews()
        except PreviewException, e:
            preview.cleanup()
            file_obj.close()
            abort(e, params['url'], callback)
        else:
            preview.cleanup()
            file_obj.close()

        base_document_url = storage.get_base_document_url()
        docviewer_url = os.path.join(base_document_url, DOCVIEWER_SUFFIX)

        print u"Document previewed in %s" % docviewer_url\
            .replace('{size}', 'normal').replace('{page}', '1')

        if callback:
            requests.post(params['callback'],
                          verify=False,
                          data={'success': True, 'num_pages': preview.pages,
                                'docviewer_url': docviewer_url})


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import os.path
try:
    import settings
except ImportError:
    settings = None

import mimetypes

mimetypes.add_type('application/x-tika-ooxml', '.docx')

REDIS_HOST = getattr(settings, 'REDIS_HOST', 'localhost')
REDIS_PORT = getattr(settings, 'REDIS_PORT', 6379)
REDIS_DB = getattr(settings, 'REDIS_DB', 0)
DEFAULT_REDIS_QUEUE_KEY = getattr(settings, 'DEFAULT_REDIS_QUEUE_KEY',
                                  'insight-reloaded')
REDIS_QUEUE_KEYS = getattr(settings, 'REDIS_QUEUE_KEYS', [])

if DEFAULT_REDIS_QUEUE_KEY not in REDIS_QUEUE_KEYS:
    REDIS_QUEUE_KEYS.append(DEFAULT_REDIS_QUEUE_KEY)


ALLOWED_EXTENSIONS = getattr(settings, 'ALLOWED_EXTENSIONS',
                             ['.pdf', '.jpeg', '.jpg', '.doc', '.docx', '.xls',
                              '.xlsx', '.odt', '.ods', '.ppt', '.pptx', '.odp',
                              '.png', '.gif', '.txt'])

PREVIEW_SIZES = getattr(settings, 'PREVIEW_SIZES',
                        {'150': 'small', '750': 'normal', '1000': 'large'})
CROP_SIZE = getattr(settings, "CROP_SIZE", 12)  # In percents (%)

PREFIX_URL = getattr(settings, 'PREFIX_URL', 'http://localhost/viewer_cache')
DOCVIEWER_SUFFIX = getattr(settings, 'DOCVIEWER_SUFFIX',
                           'document_{size}_p{page}.png')
TEMP_DIRECTORY = getattr(settings, 'TEMP_DIRECTORY', '/tmp')
DESTINATION_ROOT = getattr(settings, 'DESTINATION_ROOT',
                           os.path.join(TEMP_DIRECTORY, 'previews'))

SENTRY_DSN = getattr(settings, 'SENTRY_DSN', None)

STORAGE_CLASS = getattr(settings, 'STORAGE_CLASS',
    'insight_reloaded.storage.file_system:FileSystemStorage') # FS or S3

S3_ACCESS_KEY = getattr(settings, 'S3_ACCESS_KEY', None)
S3_SECRET_KEY = getattr(settings, 'S3_SECRET_KEY', None)
S3_BUCKET_NAME = getattr(settings, 'S3_BUCKET_NAME', 'insight-previews-test')



from os import makedirs
from os.path import join
from shutil import rmtree, move
import hashlib
from insight_reloaded.insight_settings import (DESTINATION_ROOT, PREFIX_URL)


class FileSystemStorage(object):

    def __init__(self, document_url, destination_root=None, prefix_url=None):
        if not destination_root:
            destination_root = DESTINATION_ROOT
        self.destination_root = destination_root

        if not prefix_url:
            prefix_url = PREFIX_URL
        self.prefix_url = prefix_url

        self.document_url = document_url

    def prepare(self):
        """Create a unique identifier for the document, create the path
        and return it.
        """
        doc_uuid = hashlib.sha1(self.document_url).hexdigest()
        document_path = join(self.destination_root, string_to_folder_path(doc_uuid))
        try:
            makedirs(document_path)
        except OSError:
            print "%s already exists." % document_path
        self.destination_folder = document_path

    def get_path(self, path):
        return join(self.destination_folder, path)

    def save(self, path, filename):
        move(path, join(self.destination_folder, filename))

    def get_base_document_url(self):
        return self.destination_folder.replace(self.destination_root,
            self.prefix_url)


def string_to_folder_path(s):
    """Split a string into 2-char length folders.

    >>> string_to_folder_path('3614816AA000002781')
    '361/481/6AA/000/002/781'

    """
    if s:
        size = len(s) / 3 * 3
        folders = [s[i:i + 3] for i in range(0, size, 3)]
        return join(*folders)
    return ""



from os.path import join
import hashlib

from boto.s3.connection import S3Connection, Location
from boto.s3.key import Key
from boto.exception import S3ResponseError
from insight_reloaded.insight_settings import (S3_ACCESS_KEY, S3_SECRET_KEY,
    S3_BUCKET_NAME)


class S3Storage(object):

    def __init__(self, document_url, s3_location=None):
        self.document_url = document_url
        self.document_hash = hashlib.sha1(self.document_url).hexdigest()
        self.conn = S3Connection(S3_ACCESS_KEY, S3_SECRET_KEY)
        self.paths = {}

        if s3_location is None:
            s3_location = Location.EU

        try:
            self.bucket = self.conn.get_bucket(S3_BUCKET_NAME)
        except S3ResponseError, e:
            if e.status != 404:
                raise

            self.bucket = self.conn.create_bucket(self.bucket, location=s3_location)

    def prepare(self):
        """Do nothing
        """
        pass

    def get_path(self, filename):
        return self.paths[filename]

    def save(self, path, filename):
        self.paths[filename] = path
        key = Key(self.bucket)
        key.storage_class = 'REDUCED_REDUNDANCY'
        key.key = join(self.document_hash, filename)
        key.set_contents_from_filename(path)

    def get_base_document_url(self):
        return self.document_hash



# -*- coding: utf-8 -*-
"""Python packaging."""
from os.path import abspath, dirname, join
from setuptools import setup, find_packages


def read_relative_file(filename):
    """Returns contents of the given file, which path is supposed relative
    to this module."""
    with open(join(dirname(abspath(__file__)), filename)) as f:
        return f.read()

name = 'insight_reloaded'
version = read_relative_file('VERSION').strip()
readme = read_relative_file('README')
requirements = ['setuptools', 'tornado', 'tornado-redis',
                'requests', 'redis', 'raven', 'boto']
entry_points = {'console_scripts':
                ['insight_api = insight_reloaded.api:main',
                 'insight = insight_reloaded.worker:main',
                 'fake_callback = insight_reloaded.fake_callback:main',
                 ]
                }


if __name__ == '__main__':  # ``import setup`` doesn't trigger setup().
    setup(name=name,
          version=version,
          description="""
          A full async docsplit previewer server based on Tornado and Redis""",
          long_description=readme,
          classifiers=["Programming Language :: Python",
                       'License :: Other/Proprietary License',
                       ],
          keywords='',
          author='Novapost',
          author_email='rd@novapost.fr',
          url='https://github.com/novagile/%s' % name,
          license='closed source',
          packages=find_packages(),
          include_package_data=True,
          zip_safe=False,
          install_requires=requirements,
          entry_points=entry_points,
          )



import tornado.ioloop
import tornado.web


class MainHandler(tornado.web.RequestHandler):
    def post(self):
        print self.request.arguments
        self.write(self.request.arguments)

application = tornado.web.Application([
    (r"/", MainHandler),
])


def main():
    application.listen(55555)
    print "Listening callback on 55555"
    tornado.ioloop.IOLoop.instance().start()

if __name__ == '__main__':
    main()
